# Generated by roxygen2: do not edit by hand

S3method(tidybulk::filterByExpr,DGEList)
S3method(tidybulk::filterByExpr,default)
S3method(validation,default)
S3method(validation,tidybulk)
export(adjust_abundance)
export(aggregate_duplicates)
export(as_SummarizedExperiment)
export(as_matrix)
export(cluster_elements)
export(deconvolve_cellularity)
export(describe_transcript)
export(fill_missing_abundance)
export(get_X_cibersort)
export(get_bibliography)
export(identify_abundant)
export(impute_missing_abundance)
export(keep_abundant)
export(keep_variable)
export(log10_reverse_trans)
export(logit_trans)
export(pivot_sample)
export(pivot_transcript)
export(quantile_normalise_abundance)
export(reduce_dimensions)
export(remove_redundancy)
export(resolve_complete_confounders_of_non_interest)
export(rotate_dimensions)
export(scale_abundance)
export(scale_x_log10_reverse)
export(scale_y_log10_reverse)
export(test_differential_abundance)
export(test_differential_expression)
export(test_gene_enrichment)
export(test_gene_overrepresentation)
export(test_gene_rank)
exportMethods(as_SummarizedExperiment)
import(lifecycle)
import(tibble)
importFrom(GenomicRanges,makeGRangesListFromDataFrame)
importFrom(Matrix,colSums)
importFrom(Matrix,rowMeans)
importFrom(Matrix,t)
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,as.data.frame)
importFrom(S4Vectors,metadata)
importFrom(SummarizedExperiment,"assays<-")
importFrom(SummarizedExperiment,"colData<-")
importFrom(SummarizedExperiment,"rowData<-")
importFrom(SummarizedExperiment,"rowRanges<-")
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,as.data.frame)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assayNames)
importFrom(SummarizedExperiment,assays)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,colnames)
importFrom(SummarizedExperiment,rowData)
importFrom(SummarizedExperiment,rowRanges)
importFrom(crayon,blue)
importFrom(dplyr,across)
importFrom(dplyr,add_count)
importFrom(dplyr,arrange)
importFrom(dplyr,as_tibble)
importFrom(dplyr,between)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,count)
importFrom(dplyr,desc)
importFrom(dplyr,distinct)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,if_else)
importFrom(dplyr,inner_join)
importFrom(dplyr,join_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_if)
importFrom(dplyr,n)
importFrom(dplyr,n_distinct)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(dplyr,rowwise)
importFrom(dplyr,select)
importFrom(dplyr,select_if)
importFrom(dplyr,setdiff)
importFrom(dplyr,slice)
importFrom(dplyr,starts_with)
importFrom(dplyr,summarise)
importFrom(dplyr,summarise_all)
importFrom(dplyr,ungroup)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(lifecycle,deprecate_soft)
importFrom(lifecycle,deprecate_warn)
importFrom(magrittr,"%$%")
importFrom(magrittr,divide_by)
importFrom(magrittr,equals)
importFrom(magrittr,extract2)
importFrom(magrittr,multiply_by)
importFrom(magrittr,not)
importFrom(magrittr,set_colnames)
importFrom(magrittr,set_rownames)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,slot)
importFrom(parallel,clusterExport)
importFrom(parallel,makeCluster)
importFrom(parallel,mclapply)
importFrom(parallel,parLapply)
importFrom(parallel,stopCluster)
importFrom(purrr,map)
importFrom(purrr,map2)
importFrom(purrr,map2_dfc)
importFrom(purrr,map2_dfr)
importFrom(purrr,map_chr)
importFrom(purrr,map_dfr)
importFrom(purrr,map_int)
importFrom(purrr,reduce)
importFrom(purrr,when)
importFrom(rlang,"!!")
importFrom(rlang,":=")
importFrom(rlang,enquo)
importFrom(rlang,inform)
importFrom(rlang,is_quosure)
importFrom(rlang,quo)
importFrom(rlang,quo_is_missing)
importFrom(rlang,quo_is_null)
importFrom(rlang,quo_is_symbol)
importFrom(rlang,quo_is_symbolic)
importFrom(rlang,quo_name)
importFrom(rlang,quo_set_env)
importFrom(rlang,quo_squash)
importFrom(rlang,set_names)
importFrom(scales,extended_breaks)
importFrom(scales,label_scientific)
importFrom(scales,trans_new)
importFrom(stats,AIC)
importFrom(stats,anova)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,dist)
importFrom(stats,hat)
importFrom(stats,kmeans)
importFrom(stats,lm)
importFrom(stats,logLik)
importFrom(stats,lsfit)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(stats,plogis)
importFrom(stats,prcomp)
importFrom(stats,predict)
importFrom(stats,qlogis)
importFrom(stats,rnorm)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,terms)
importFrom(stats,update.formula)
importFrom(stats,vcov)
importFrom(stringr,str_c)
importFrom(stringr,str_detect)
importFrom(stringr,str_remove)
importFrom(stringr,str_replace)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_subset)
importFrom(tibble,as_tibble)
importFrom(tibble,enframe)
importFrom(tibble,rowid_to_column)
importFrom(tibble,tibble)
importFrom(tidyr,drop_na)
importFrom(tidyr,gather)
importFrom(tidyr,nest)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,spread)
importFrom(tidyr,unite)
importFrom(tidyr,unnest)
importFrom(tidyselect,all_of)
importFrom(tidyselect,any_of)
importFrom(tidyselect,ends_with)
importFrom(utils,capture.output)
importFrom(utils,combn)
importFrom(utils,data)
importFrom(utils,head)
importFrom(utils,install.packages)
importFrom(utils,packageDescription)
importFrom(utils,packageVersion)
importFrom(utils,tail)
