#' Cibersort reference
#' 
#' @references Newman, A. M., Liu, C. L., Green, M. R., Gentles, A. J., Feng, W., Xu, Y., Hoang, C. D., Diehn, M., & Alizadeh, A. A. (2015). Robust enumeration of cell subsets from tissue expression profiles. Nature Methods, 12(5), 453–457. https://doi.org/10.1038/nmeth.3337
#'
"X_cibersort"

#' Needed for vignette vignette_manuscript_signature_tsne
#'
"vignette_manuscript_signature_tsne"

#' Needed for vignette vignette_manuscript_signature_boxplot
#'
"vignette_manuscript_signature_boxplot"

#' Needed for vignette vignette_manuscript_signature_tsne2
#'
"vignette_manuscript_signature_tsne2"

#' Needed for tests tximeta_summarizeToGene_object, It is SummarizedExperiment from tximeta
#'
"tximeta_summarizeToGene_object"
