% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test_stratification_cellularity.R
\docType{methods}
\name{test_stratification_cellularity,SummarizedExperiment-method}
\alias{test_stratification_cellularity,SummarizedExperiment-method}
\alias{test_stratification_cellularity,RangedSummarizedExperiment-method}
\title{test_stratification_cellularity}
\usage{
\S4method{test_stratification_cellularity}{SummarizedExperiment}(
  .data,
  .formula,
  .abundance = NULL,
  method = c("cibersort", "llsr", "epic", "mcp_counter", "quantiseq", "xcell"),
  reference = X_cibersort,
  ...
)

\S4method{test_stratification_cellularity}{RangedSummarizedExperiment}(
  .data,
  .formula,
  .abundance = NULL,
  method = c("cibersort", "llsr", "epic", "mcp_counter", "quantiseq", "xcell"),
  reference = X_cibersort,
  ...
)
}
\arguments{
\item{.data}{A `SummarizedExperiment` object}

\item{.formula}{A formula representing the desired linear model}

\item{.abundance}{The name of the transcript/gene abundance column}

\item{method}{A character string naming the deconvolution method}

\item{reference}{Reference matrix or method-specific handle}

\item{...}{Additional arguments passed through to the underlying deconvolution function}
}
\value{
A consistent object (to the input) with additional columns for the statistics from the hypothesis test (e.g.,  log fold change, p-value and false discovery rate).

A consistent object (to the input) with additional columns for the statistics from the hypothesis test (e.g.,  log fold change, p-value and false discovery rate).
}
\description{
test_stratification_cellularity

test_stratification_cellularity
}
